-- =====================================================================
-- Copyright  2010-2011 by Cryptographic Engineering Research Group (CERG),
-- ECE Department, George Mason University
-- Fairfax, VA, U.S.A.
-- =====================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;		
use work.sha3_pkg.all;
use work.sha3_blake_package.all;


entity gfunc_half_modified is
	generic ( 	iw : integer := 32; 	 
				h : integer := 256 );
	port(
		ain : in std_logic_vector(iw-1 downto 0);
		bin : in  std_logic_vector(iw-1 downto 0);
      	cin : in  std_logic_vector(iw-1 downto 0);
      	din : in  std_logic_vector(iw-1 downto 0);      
		const : in  std_logic_vector(iw-1 downto 0);	
		sel : in std_logic;
      	aout : out std_logic_vector(iw-1 downto 0);
      	bout : out std_logic_vector(iw-1 downto 0);
      	cout : out std_logic_vector(iw-1 downto 0);
      	dout : out std_logic_vector(iw-1 downto 0)
		);		 
end gfunc_half_modified;

--}} end of automatically maintained section

architecture struct of gfunc_half_modified is
	signal csa_a1_1, csa_a1_2, csa_a2_1, csa_a2_2 : std_logic_vector(iw-1 downto 0);

	signal  aadd, dxor, dxor_rot, bxor, bxor_rot, cadd    :   std_logic_vector (iw-1 downto 0);
begin 
	aadd <= const + ain + bin;
	dxor <= din xor aadd;					 
	
	cadd <= dxor_rot + cin;
	bxor <= cadd xor bin;	
	
	-- output data
	aout   <= aadd;
	bout   <= bxor_rot;
	cout   <= cadd;
	dout   <= dxor_rot;
	
	arch_32 : if h <= 256 generate		
		dxor_rot <= rolx(dxor, 32-g_rot_arch32(0)) when sel = '0' else rolx(dxor, 32-g_rot_arch32(2));
		bxor_rot <= rolx(bxor, 32-g_rot_arch32(1)) when sel = '0' else rolx(bxor, 32-g_rot_arch32(3));
	end generate;
	
	arch_64 : if h > 256 generate
		dxor_rot <= rolx(dxor, 64-g_rot_arch64(0)) when sel = '0' else rolx(dxor, 64-g_rot_arch64(2));
		bxor_rot <= rolx(bxor, 64-g_rot_arch64(1)) when sel = '0' else rolx(bxor, 64-g_rot_arch64(3));		
	end generate;	
end struct;
